from ftw.topics import _
from ftw.topics.interfaces import ITopicReferencePresentation
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface


class DefaultRepresentation(object):
    """Default representation"""
    implements(ITopicReferencePresentation)
    adapts(Interface, Interface)

    template = ViewPageTemplateFile(
        'templates/default_representation.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.objects = []

    def consume(self, objects):
        """Consume all objects left"""
        self.objects = list(objects)

    def render(self):
        return self.template()

    def title(self):
        return _(u'label_further_content', default=u'Further content')

    def position(self):
        return 1000

    def available(self):
        return bool(self.objects)



from ftw.topics.interfaces import ITopicTree
from plone.dexterity.content import Container
from plone.directives.form import Schema
from zope.interface import implements


class ITopicTreeSchema(Schema):
    pass


class TopicTree(Container):
    implements(ITopicTree)



from ftw.simplelayout.contenttypes.contents.interfaces import IContentPage
from ftw.topics import _
from ftw.topics.browser.representation import DefaultRepresentation
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class ContentPageRepresentation(DefaultRepresentation):

    template = ViewPageTemplateFile(
        'contentpage_representation.pt')

    def consume(self, objects):
        """Consume all objects left"""

        for obj in objects:
            if IContentPage.providedBy(obj):
                self.objects.append(obj)
            else:
                yield obj

    def title(self):
        return _(u'label_contentpage', default=u'Content pages')

    def position(self):
        return 100



from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from ftw.topics.interfaces import ITopic


def get_brain_data(brain):
    return {'title': brain.Title,
            'path': brain.getPath(),
            'url': brain.getURL(),
            'children': []}


def get_parent_path(path):
    return '/'.join(path.rstrip('/').split('/')[:-1])


def make_treeish(data):
    path2node = dict(map(lambda item: (item['path'], item), data))

    tree = []
    for node in data:
        parent_path = get_parent_path(node['path'])
        if parent_path in path2node:
            path2node[parent_path]['children'].append(node)
        else:
            tree.append(node)

    return tree


class TreeView(BrowserView):

    def get_tree(self):
        brains = self._get_brains()
        data = map(get_brain_data, brains)
        data = sorted(data, key=lambda item: item.get('title'))
        data = make_treeish(data)
        return data

    def _get_brains(self):
        query = {'object_provides': ITopic.__identifier__,
                 'path': {'query': '/'.join(self.context.getPhysicalPath()),
                          'depth': 2}}

        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(query)



from DateTime import DateTime
from ftw.topics.interfaces import IBackReferenceCollector
from ftw.topics.testing import EXAMPLE_CONTENT_DEFAULT_FUNCTIONAL
from plone.uuid.interfaces import IUUID
from unittest import TestCase
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface.verify import verifyObject


class TestDefaultCollector(TestCase):

    layer = EXAMPLE_CONTENT_DEFAULT_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.subsite = self.portal.get('foo').get('subsite')
        self.subsite_uid = IUUID(self.subsite)
        self.empty_subsite = self.portal.get('empty-subsite')
        self.empty_subsite_uid = IUUID(self.empty_subsite)

        self.tree = self.portal.get('topics')
        self.node = self.tree.get('manufacturing')
        self.subsite_tree = self.subsite.get('topics')
        self.subsite_node = self.subsite_tree.get('manufacturing')

        self.doc = self.portal.get('manufacturing-processes')
        self.subsite_doc = self.subsite.get('theories')

    def test_component_registered(self):
        self.assertTrue(
            queryMultiAdapter((self.node, None), IBackReferenceCollector),
            'No default IBackReferenceCollector adapter registered.')

    def test_component_implements_interface(self):
        comp = getMultiAdapter((self.node, None), IBackReferenceCollector)
        self.assertTrue(IBackReferenceCollector.providedBy(comp))
        verifyObject(IBackReferenceCollector, comp)

    def test_calling_returns_result(self):
        comp = getMultiAdapter((self.node, None),
                               IBackReferenceCollector)
        result = comp()

        self.assertEquals(
            [
                {'label': u'Plone site',
                 'UID': 'root',
                 'path': '/plone',
                 'objects': [self.doc],
                 'is_current_section': True},

                {'label': 'Sub Site',
                 'UID': self.subsite_uid,
                 'path': '/plone/foo/subsite',
                 'objects': [self.subsite_doc]}],
            result)

    def test_SUBSITE_calling_returns_result(self):
        comp = getMultiAdapter((self.subsite_node, None),
                               IBackReferenceCollector)
        result = comp()

        self.assertEquals(
            [
                {'label': u'Plone site',
                 'UID': 'root',
                 'path': '/plone',
                 'objects': [self.doc]},

                {'label': 'Sub Site',
                 'UID': self.subsite_uid,
                 'path': '/plone/foo/subsite',
                 'objects': [self.subsite_doc],
                 'is_current_section': True}],
            result)

    def test_get_sections(self):
        comp = getMultiAdapter((self.node, None), IBackReferenceCollector)
        self.assertEquals(
            [
                {'label': u'Plone site',
                 'UID': 'root',
                 'path': '/plone',
                 'objects': [],
                 'is_current_section': True},

                {'label': 'Sub Site',
                 'UID': self.subsite_uid,
                 'objects': [],
                 'path': '/plone/foo/subsite'},

                {'label': "Empty sub site",
                 'UID': self.empty_subsite_uid,
                 'objects': [],
                 'path': '/plone/empty-subsite'}],
            comp.get_sections())

    def test_get_brefs_per_section(self):
        comp = getMultiAdapter((self.node, None),
                               IBackReferenceCollector)
        result = comp._get_brefs_per_section()

        self.assertEquals(
            [
                {'label': u'Plone site',
                 'UID': 'root',
                 'path': '/plone',
                 'objects': [self.doc],
                 'is_current_section': True},

                {'label': 'Sub Site',
                 'UID': self.subsite_uid,
                 'path': '/plone/foo/subsite',
                 'objects': [self.subsite_doc]}],
            result)

    def test_SUBSITE_get_brefs_per_section(self):
        comp = getMultiAdapter((self.subsite_node, None),
                               IBackReferenceCollector)
        result = comp._get_brefs_per_section()

        self.assertEquals(
            [
                {'label': u'Plone site',
                 'UID': 'root',
                 'path': '/plone',
                 'objects': [self.doc]},

                {'label': 'Sub Site',
                 'UID': self.subsite_uid,
                 'path': '/plone/foo/subsite',
                 'objects': [self.subsite_doc],
                 'is_current_section': True}],
            result)

    def test_get_merged_brefs(self):
        comp = getMultiAdapter((self.node, None),
                               IBackReferenceCollector)

        result = comp._get_merged_brefs()
        self.assertIn(self.doc, result)
        self.assertIn(self.subsite_doc, result)

    def test_SUBSITE_get_merged_brefs(self):
        comp = getMultiAdapter((self.subsite_node, None),
                               IBackReferenceCollector)

        result = comp._get_merged_brefs()
        self.assertIn(self.doc, result)
        self.assertIn(self.subsite_doc, result)

    def test_get_similar_topic_objects(self):
        comp = getMultiAdapter((self.node, None),
                               IBackReferenceCollector)

        result = comp._get_similar_topic_objects()
        self.assertIn(self.node, result)
        self.assertIn(self.subsite_node, result)

    def test_SUBSITE_get_similar_topic_objects(self):
        comp = getMultiAdapter((self.subsite_node, None),
                               IBackReferenceCollector)

        result = comp._get_similar_topic_objects()
        self.assertIn(self.node, result)
        self.assertIn(self.subsite_node, result)

    def test_exclude_expired_content(self):
        comp = getMultiAdapter((self.node, None),
                               IBackReferenceCollector)

        self.assertIn(self.doc, comp._get_merged_brefs())

        self.doc.setExpirationDate(DateTime() - 10)
        self.doc.reindexObject()

        self.assertNotIn(self.doc, comp._get_merged_brefs())

    def test_exclude_future_content(self):
        comp = getMultiAdapter((self.node, None),
                               IBackReferenceCollector)

        self.assertIn(self.doc, comp._get_merged_brefs())

        self.doc.setEffectiveDate(DateTime() + 10)
        self.doc.reindexObject()

        self.assertNotIn(self.doc, comp._get_merged_brefs())



from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.testing import MockTestCase
from ftw.topics.finder import DefaultTopicTreeFinder
from ftw.topics.interfaces import ITopicRootFinder
from ftw.topics.interfaces import ITopicTree
from ftw.topics.testing import ZCML_LAYER
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import queryMultiAdapter
from zope.component.hooks import setSite
from zope.interface.verify import verifyClass


class TestDefaultTopicTreeFinder(MockTestCase):

    layer = ZCML_LAYER

    def setUp(self):
        super(TestDefaultTopicTreeFinder, self).setUp()
        self.request = self.stub_request()

    def tearDown(self):
        super(TestDefaultTopicTreeFinder, self).tearDown()
        setSite(None)

    def create_tree(self, with_trees=True):
        objs = {}

        objs['site'] = self.providing_stub(IPloneSiteRoot, INavigationRoot)
        objs['site'].getPhysicalPath.return_value = ['', 'site']
        objs['site'].getSiteManager.side_effect = getSiteManager

        if with_trees:
            objs['root'] = self.providing_stub(ITopicRootFinder)
            objs['site'].contentValues.return_value = [objs['root']]
        else:
            objs['site'].contentValues.return_value = []

        objs['foo'] = self.stub()
        objs['foo'].getPhysicalPath.return_value = ['', 'site', 'foo']
        self.set_parent(objs['foo'], objs['site'])

        objs['subsite'] = self.providing_stub(INavigationRoot)
        objs['subsite'].getPhysicalPath.return_value = ['', 'site', 'subsite']
        self.set_parent(objs['subsite'], objs['site'])

        objs['bar'] = self.stub()
        objs['bar'].getPhysicalPath.return_value = [
            '', 'site', 'subsite', 'bar']
        self.set_parent(objs['bar'], objs['subsite'])

        if with_trees:
            objs['subtree'] = self.providing_stub(ITopicTree)
            objs['subsite'].contentValues.return_value = [objs['subtree']]
        else:
            objs['subsite'].contentValues.return_value = []

        return objs

    def test_adapter_implements_interface(self):
        self.assertTrue(
            ITopicRootFinder.implementedBy(DefaultTopicTreeFinder))
        verifyClass(ITopicRootFinder, DefaultTopicTreeFinder)

    def test_adapter_registered(self):
        context = self.create_dummy()

        self.assertTrue(
            queryMultiAdapter((context, self.request), ITopicRootFinder),
            'ZugTopicTreeFinder adapter is not registered')

    def test_finds_next_navigation_root(self):
        objs = self.create_tree(with_trees=True)

        obj = objs['foo']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')

    def test_subsite_obj_finds_subsite_with_trees(self):
        objs = self.create_tree(with_trees=True)

        obj = objs['bar']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site/subsite')

    def test_subsite_obj_finds_site_without_trees(self):
        objs = self.create_tree(with_trees=False)

        obj = objs['bar']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')

    def test_finds_site_on_site(self):
        objs = self.create_tree(with_trees=True)

        obj = objs['site']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')

    def test_fall_back_to_plone_site_root(self):
        site = self.create_dummy(getSiteManager=getSiteManager,
                                 getPhysicalPath=lambda: ('', 'site'))
        setSite(site)

        finder = getMultiAdapter((self.create_dummy(), self.request),
                                 ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')



from plone import api
import transaction


def add_behaviors(type_to_configure, *additional_behaviors):
    fti = api.portal.get().portal_types.get(type_to_configure)
    behaviors = list(fti.behaviors)
    behaviors += list(additional_behaviors)
    fti.behaviors = tuple(set(behaviors))
    transaction.commit()



from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.topics.interfaces import IBackReferenceCollector
from ftw.topics.interfaces import ITopic
from ftw.topics.interfaces import ITopicReferencePresentation
from zope.component import getAdapters
from zope.component import getMultiAdapter


class TopicListing(object):

    listing_template = ViewPageTemplateFile('templates/topic_listing.pt')

    def __init__(self):
        self.sections = None
        self.objects = None
        self.representations = None

    def render_topic_listing(self):
        self.sections, self.objects = self._get_sections_and_objects()
        self.representations = self._get_representations()

        return self.listing_template()

    def get_child_topics(self):
        query = {'object_provides': ITopic.__identifier__,
                 'path': {'query': '/'.join(self.context.getPhysicalPath()),
                          'depth': 1},
                 'sort_on': 'sortable_title'}

        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(query)

    def has_multiple_sections(self):
        collector = getMultiAdapter((self.context, self.request),
                                    IBackReferenceCollector)

        return len(collector.get_sections()) > 1

    def topic_or_similar_topic_has_brefs(self):
        for section in self.sections:
            if len(section.get('objects')) > 0:
                return True
        return False

    def display_section_headings(self):
        return self.has_multiple_sections() and \
            self.topic_or_similar_topic_has_brefs()

    def _get_representations(self):
        adapters = dict(getAdapters((self.context, self.request),
                                     ITopicReferencePresentation)).values()

        adapters.sort(key=lambda adapter: adapter.position())

        objects = self.objects
        for adapter in adapters:
            objects = adapter.consume(objects)

        return adapters

    def _get_sections_and_objects(self):
        collector = getMultiAdapter((self.context, self.request),
                                    IBackReferenceCollector)
        sections = collector()
        if not sections:
            return [], []

        section = self._get_selected_section(sections)
        section['selected'] = True

        section['objects'].sort(key=lambda obj: obj.Title())

        return (sections, section['objects'])

    def _get_selected_section(self, sections):
        selected_section = self.request.get('section', None)
        for section in sections:
            if section.get('UID') == selected_section:
                return section

        return self._get_current_section(sections)

    def _get_current_section(self, sections):
        sections_by_path = dict(map(lambda item: (item.get('path'), item),
                                    sections))
        path = '/'.join(self.context.getPhysicalPath())

        # Select all sections where the current context is in.
        # This may be multiple sections if nested.
        parental_section_paths = [spath for spath in sections_by_path.keys()
                                  if path.startswith(spath)]

        # Sort the paths by length and pick the longest one, this is
        # the nearest parent.
        section_path = sorted(parental_section_paths,
                              key=len, reverse=True)[0]
        return sections_by_path[section_path]


class TopicView(BrowserView, TopicListing):

    def __init__(self, *args, **kwargs):
        BrowserView.__init__(self, *args, **kwargs)
        TopicListing.__init__(self)

    def subtopics(self):
        return self.render_topic_listing()



import os
from setuptools import setup, find_packages


version = '2.1.1'


extras_require = {}


tests_require = [
    'ftw.simplelayout [contenttypes]',
    'ftw.testing',
    'ftw.testbrowser',
    'ftw.builder',

    'zope.i18n',
    'z3c.autoinclude',
    'transaction',
    'zope.traversing',
    'AccessControl',
    'zope.configuration',

    'plone.uuid',
    'plone.testing',
    'plone.app.testing',
    'plone.app.contenttypes',

    'ftw.inflator [dexterity]',
    'Products.DateRecurringIndex',
    ]

extras_require['tests'] = tests_require


setup(name='ftw.topics',
      version=version,
      description='Create subject trees in Plone',

      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw topics',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.topics',

      license='GPL2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'setuptools',

        'zope.schema',
        'zope.interface',
        'zope.component',
        'zope.i18nmessageid',
        'Acquisition',
        'Zope2',
        'zc.relation',

        'Plone',
        'plone.app.layout',
        'plone.browserlayer',
        'plone.memoize',
        'Products.GenericSetup',
        'Products.CMFCore',
        'Products.CMFPlone',

        'plone.autoform',
        'plone.behavior',
        'plone.dexterity',
        'plone.app.dexterity',
        'plone.app.relationfield',
        'collective.dexteritytextindexer',
        'collective.geo.openlayers',
        'collective.geo.mapwidget',
        'plone.app.referenceablebehavior',
        'plone.directives.form',

        'ftw.upgrade',
        'ftw.referencewidget',
        ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from ftw.testbrowser import browsing
from ftw.topics.interfaces import IBackReferenceCollector
from ftw.topics.testing import EXAMPLE_CONTENT_DEFAULT_FUNCTIONAL
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.dexterity.fti import DexterityFTI
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from unittest import TestCase
from zope.component import getMultiAdapter
import transaction


class TestTopicSupportBehavior(TestCase):

    layer = EXAMPLE_CONTENT_DEFAULT_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        portal_types = getToolByName(self.portal, 'portal_types')

        fti = DexterityFTI('DxTopicSupport')
        fti.behaviors = (
            'plone.app.dexterity.behaviors.metadata.IBasic',
            'plone.app.content.interfaces.INameFromTitle',
            'ftw.topics.behavior.ITopicSupportSchema',)
        portal_types._setObject('DxTopicSupport', fti)
        fti.lookupSchema()

        transaction.commit()

        self.browser = Browser(self.layer['app'])
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                TEST_USER_NAME, TEST_USER_PASSWORD,))
        self.browser.handleErrors = False

    def test_behavior(self):
        self.browser.open(self.portal.portal_url() + '/++add++DxTopicSupport')
        browse_field_name = 'form.widgets.ITopicSupportSchema.topics'

        self.browser.getControl(label='Title').value = 'My Object'
        self.browser.getControl(name=browse_field_name).value = '/plone/' \
            'topics/manufacturing/quality'
        self.browser.getControl(label='Save').click()

        obj = self.portal.get('my-object')
        agile = self.portal.get('topics').get('manufacturing').get(
            'quality')

        collector = getMultiAdapter((agile, self.request),
                                    IBackReferenceCollector)
        section, = collector()
        self.assertItemsEqual((obj,), section.get('objects'))

    @browsing
    def test_write_permission(self, browser):
        # Create a content item having topic references.
        browser.login().open(self.portal.portal_url() + '/++add++DxTopicSupport')
        browser.fill({
            'Title': u'My Object',
            'form.widgets.ITopicSupportSchema.topics': '/plone/topics/manufacturing/quality',
        }).save()
        # The user sees the topic field on the edit form.
        obj = self.portal.get('my-object')
        browser.visit(obj, view='edit')
        expected = sorted([
            'form.widgets.IBasic.description',
            'form.widgets.IBasic.title',
            'form.widgets.ITopicSupportSchema.topics',
            'form.buttons.save',
            'form.buttons.cancel',
            ])
        actual = sorted([field for field in browser.forms['form'].fields.keys()
                         if field in expected])

        self.assertEqual(expected, actual)

        # Remove the permission to set topic references.
        self.portal.manage_permission('ftw.topics: Set Topic Reference', roles=[], acquire=False)
        transaction.commit()
        # The user no longer sees the topic field on the edit form.
        browser.login().visit(obj, view='edit')
        expected2 = sorted([
            'form.widgets.IBasic.description',
            'form.widgets.IBasic.title',
            'form.buttons.save',
            'form.buttons.cancel',
            ])
        actual2 = sorted([field for field in browser.forms['form'].fields.keys()
                         if field in expected])

        # Make sure anything else than the behavior is still there
        self.assertEqual(expected2, actual2)

        # Test that behavior is not there anymore
        self.assertNotIn(
            'form.widgets.ITopicSupportSchema.topics',
            browser.forms['form'].fields.keys())



from ftw.testbrowser import browsing
from ftw.topics.interfaces import ITopic
from ftw.topics.interfaces import ITopicTree
from ftw.topics.testing import TOPICS_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from unittest import TestCase
import transaction


class TestContentTypeCreation(TestCase):

    layer = TOPICS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        transaction.commit()

    @browsing
    def test_create_topic_tree(self, browser):
        browser.login()
        browser.open(self.portal)
        browser.css('#ftw-topics-topictree').first.click()

        self.assertTrue(
            browser.url.endswith('++add++ftw.topics.TopicTree'),
            'Should be on the topic tree add view, but url is: %s' % (
                browser.url))

        browser.fill({'Title': 'Topical'}).submit()

        self.assertTrue(
            browser.url.endswith('topical/view'),
            'Should be on "topical" view, but url is: %s' % (
                browser.url))
        self.assertIn('Topical', browser.contents)

        tree = self.portal.get('topical')
        self.assertTrue(ITopicTree.providedBy(tree))

    @browsing
    def test_create_topic(self, browser):
        # first create a tree
        browser.login()
        browser.open('http://nohost/plone/++add++ftw.topics.TopicTree')
        browser.fill({'Title': 'Topical'}).submit()

        self.assertEqual(browser.url,
                         'http://nohost/plone/topical/view')

        factory_link = browser.css('#ftw-topics-topic').first
        self.assertTrue(
            factory_link,
            'There is no "Topic" factory link on the topic tree view.')
        factory_link.click()

        self.assertEqual(
            browser.url,
            'http://nohost/plone/topical/++add++ftw.topics.Topic')

        browser.fill({'Title': 'Manufacturing'}).submit()

        self.assertEqual(browser.url,
                         'http://nohost/plone/topical/manufacturing/view')
        self.assertIn('Manufacturing', browser.contents)

        topic = self.portal.get('topical').get('manufacturing')
        self.assertTrue(ITopic.providedBy(topic))
